## -*- texinfo -*-
## @deftypefn  {Function File} {} imapplymatrix ( @var{M},@var{X})
## @deftypefnx {Function File} {} imapplymatrix ( @var{M},@var{X},@var{C}) 
## @deftypefnx {Function File} {} imapplymatrix (@dots{}, @var{output_type})
## @end deftypefn

function Y = imapplymatrix(M,X)
  
  if (nargin < 2 || nargin > 4)
    print_usage ();
   elseif (nargin>2)
    error("Currently, only imapplymatrix(M,X) is implemented \n") 
   elseif (!Xmatrix (X))
    error("The second input must be an m-by-n-by-p numeric array \n");
   elseif  (!Mmatrix (M,X))
    error("The second input must be a q-by-p numeric array \n");
  endif
  [m,n,p] = size(X);
  q = size(M,1);
  I = im2double(X);
  % Make the combination
  XI = I(:,:,1)(:)+0.01;
  YI = I(:,:,2)(:)+0.01;
  ZI = I(:,:,3)(:)+0.01;
  Y = M*[XI YI ZI]';
  % reshape the output matrix;
  Y = reshape(Y',m,n,q);
  % Return image 
  switch class(X)
    case 'uint8'
      Y = Y*255;
    case 'uint16'
      Y = Y*256^2;
    case 'uint32'
      Y = Y*256^3;
  endswitch
  
  Y = cast(Y,class(X));
  
endfunction

function check = Mmatrix(M,X)
   p = size(M,2);
   q = size(M,1);
   %Types={'single ',' double ',' int8 ',' int16 ',' int32 ',' uint8 ',' uint16 ',' uint32 '};
 
   check  = (isnumeric(M) && ! issparse (M) && ! isempty (M) && isreal (M) && ndims(M)==2 && p==size(X,3) && q<=p);
endfunction

function check = Xmatrix(X)
    check  = (isnumeric (X) && ! issparse (X) && ! isempty (X) && isreal (X) && ndims(X)==3);

endfunction

